@@ -1,5 +1,4 @@
 /*
-#ifdef LINUX
  * Copyright (c) 1991, 1992 Paul Kranenburg <pk@cs.few.eur.nl>
  * Copyright (c) 1993 Branko Lankester <branko@hacktic.nl>
  * Copyright (c) 1993, 1994, 1995, 1996 Rick Sladkey <jrs@world.std.com>
@@ -28,7 +27,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: file.c,v 1.93 2008/08/01 01:13:10 roland Exp $
+ *	$Id$
  */
 
 #include "defs.h"
@@ -239,7 +238,7 @@ const struct xlat open_mode_flags[] = {
 #ifdef O_RSYNC
 	{ O_RSYNC,	"O_RSYNC"	},
 #endif
-#ifdef O_NDELAY
+#if defined(O_NDELAY) && (O_NDELAY != O_NONBLOCK)
 	{ O_NDELAY,	"O_NDELAY"	},
 #endif
 #ifdef O_PRIV
@@ -328,34 +327,75 @@ const struct xlat open_mode_flags[] = {
 # define AT_FDCWD                -100
 #endif
 
+/* The fd is an "int", so when decoding x86 on x86_64, we need to force sign
+ * extension to get the right value.  We do this by declaring fd as int here.
+ */
 static void
-print_dirfd(long fd)
+print_dirfd(int fd)
 {
 	if (fd == AT_FDCWD)
 		tprintf("AT_FDCWD, ");
 	else
-		tprintf("%ld, ", fd);
+		tprintf("%d, ", fd);
 }
 #endif
 
+/*
+ * Pity stpcpy() is not standardized...
+ */
+static char *
+str_append(char *dst, const char *src)
+{
+	while ((*dst = *src++) != '\0')
+		dst++;
+	return dst;
+}
+
 /*
  * low bits of the open(2) flags define access mode,
  * other bits are real flags.
  */
-void
-tprint_open_modes(struct tcb *tcp, mode_t flags)
+const char *
+sprint_open_modes(mode_t flags)
 {
-	const char *str = xlookup(open_access_modes, flags & 3);
+	static char outstr[1024];
+	char *p;
+	char sep = 0;
+	const char *str;
+	const struct xlat *x;
 
-	if (str)
-	{
-		tprintf("%s", str);
+	p = str_append(outstr, "flags ");
+	str = xlookup(open_access_modes, flags & 3);
+	if (str) {
+		p = str_append(p, str);
 		flags &= ~3;
 		if (!flags)
-			return;
-		tprintf("|");
+			return outstr;
+		sep = '|';
 	}
-	printflags(open_mode_flags, flags, "O_???");
+
+	for (x = open_mode_flags; x->str; x++) {
+		if ((flags & x->val) == x->val) {
+			if (sep)
+				*p++ = sep;
+			p = str_append(p, x->str);
+			flags &= ~x->val;
+			if (!flags)
+				return outstr;
+			sep = '|';
+		}
+	}
+	/* flags is still nonzero */
+	if (sep)
+		*p++ = sep;
+	sprintf(p, "%#x", flags);
+	return outstr;
+}
+
+void
+tprint_open_modes(mode_t flags)
+{
+	tprintf("%s", sprint_open_modes(flags) + sizeof("flags"));
 }
 
 static int
@@ -365,7 +405,7 @@ decode_open(struct tcb *tcp, int offset)
 		printpath(tcp, tcp->u_arg[offset]);
 		tprintf(", ");
 		/* flags */
-		tprint_open_modes(tcp, tcp->u_arg[offset + 1]);
+		tprint_open_modes(tcp->u_arg[offset + 1]);
 		if (tcp->u_arg[offset + 1] & O_CREAT) {
 			/* mode */
 			tprintf(", %#lo", tcp->u_arg[offset + 2]);
@@ -410,8 +450,7 @@ static const struct xlat openmodessol[] = {
 };
 
 int
-solaris_open(tcp)
-struct tcb *tcp;
+solaris_open(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
@@ -429,8 +468,7 @@ struct tcb *tcp;
 #endif
 
 int
-sys_creat(tcp)
-struct tcb *tcp;
+sys_creat(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
@@ -481,8 +519,7 @@ sys_faccessat(struct tcb *tcp)
 #endif
 
 int
-sys_umask(tcp)
-struct tcb *tcp;
+sys_umask(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		tprintf("%#lo", tcp->u_arg[0]);
@@ -500,8 +537,7 @@ static const struct xlat whence[] = {
 #ifndef HAVE_LONG_LONG_OFF_T
 #if defined (LINUX_MIPSN32)
 int
-sys_lseek(tcp)
-struct tcb *tcp;
+sys_lseek(struct tcb *tcp)
 {
 	long long offset;
 	int _whence;
@@ -520,8 +556,7 @@ struct tcb *tcp;
 }
 #else /* !LINUX_MIPSN32 */
 int
-sys_lseek(tcp)
-struct tcb *tcp;
+sys_lseek(struct tcb *tcp)
 {
 	off_t offset;
 	int _whence;
@@ -543,68 +578,59 @@ struct tcb *tcp;
 
 #ifdef LINUX
 int
-sys_llseek (tcp)
-struct tcb *tcp;
+sys_llseek(struct tcb *tcp)
 {
-    if (entering(tcp)) {
-	/*
-	 * This one call takes explicitly two 32-bit arguments hi, lo,
-	 * rather than one 64-bit argument for which LONG_LONG works
-	 * appropriate for the native byte order.
-	 */
-	if (tcp->u_arg[4] == SEEK_SET)
-	    tprintf("%ld, %llu, ", tcp->u_arg[0],
-		    (((long long int) tcp->u_arg[1]) << 32
-		     | (unsigned long long) (unsigned) tcp->u_arg[2]));
-	else
-	    tprintf("%ld, %lld, ", tcp->u_arg[0],
-		    (((long long int) tcp->u_arg[1]) << 32
-		     | (unsigned long long) (unsigned) tcp->u_arg[2]));
-    }
-    else {
-	long long int off;
-	if (syserror(tcp) || umove(tcp, tcp->u_arg[3], &off) < 0)
-	    tprintf("%#lx, ", tcp->u_arg[3]);
-	else
-	    tprintf("[%llu], ", off);
-	printxval(whence, tcp->u_arg[4], "SEEK_???");
-    }
-    return 0;
+	if (entering(tcp)) {
+		/*
+		 * This one call takes explicitly two 32-bit arguments hi, lo,
+		 * rather than one 64-bit argument for which LONG_LONG works
+		 * appropriate for the native byte order.
+		 */
+		if (tcp->u_arg[4] == SEEK_SET)
+			tprintf("%ld, %llu, ", tcp->u_arg[0],
+				(((long long int) tcp->u_arg[1]) << 32
+				 | (unsigned long long) (unsigned) tcp->u_arg[2]));
+		else
+			tprintf("%ld, %lld, ", tcp->u_arg[0],
+				(((long long int) tcp->u_arg[1]) << 32
+				 | (unsigned long long) (unsigned) tcp->u_arg[2]));
+	}
+	else {
+		long long int off;
+		if (syserror(tcp) || umove(tcp, tcp->u_arg[3], &off) < 0)
+			tprintf("%#lx, ", tcp->u_arg[3]);
+		else
+			tprintf("[%llu], ", off);
+		printxval(whence, tcp->u_arg[4], "SEEK_???");
+	}
+	return 0;
 }
 
 int
-sys_readahead (tcp)
-struct tcb *tcp;
+sys_readahead(struct tcb *tcp)
 {
-    if (entering(tcp)) {
-	tprintf("%ld, %lld, %ld", tcp->u_arg[0],
-# if defined LINUX_MIPSN32
-		tcp->ext_arg[1], tcp->u_arg[2]
-# elif defined IA64 || defined X86_64 || defined ALPHA || defined LINUX_MIPSN64
-		(long long int) tcp->u_arg[1], tcp->u_arg[2]
-# else
-		LONG_LONG(tcp->u_arg[1], tcp->u_arg[2]), tcp->u_arg[3]
-# endif
-		);
-    }
-    return 0;
+	if (entering(tcp)) {
+		int argn;
+		tprintf("%ld, ", tcp->u_arg[0]);
+		argn = printllval(tcp, "%lld", 1);
+		tprintf(", %ld", tcp->u_arg[argn]);
+	}
+	return 0;
 }
 #endif
 
 #if _LFS64_LARGEFILE || HAVE_LONG_LONG_OFF_T
 int
-sys_lseek64 (tcp)
-struct tcb *tcp;
+sys_lseek64(struct tcb *tcp)
 {
 	if (entering(tcp)) {
-		long long offset;
-		ALIGN64 (tcp, 1);	/* FreeBSD aligns off_t args */
-		offset = LONG_LONG(tcp->u_arg [1], tcp->u_arg[2]);
+		int argn;
+		tprintf("%ld, ", tcp->u_arg[0]);
 		if (tcp->u_arg[3] == SEEK_SET)
-			tprintf("%ld, %llu, ", tcp->u_arg[0], offset);
+			argn = printllval(tcp, "%llu, ", 1);
 		else
-			tprintf("%ld, %lld, ", tcp->u_arg[0], offset);
-		printxval(whence, tcp->u_arg[3], "SEEK_???");
+			argn = printllval(tcp, "%lld, ", 1);
+		printxval(whence, tcp->u_arg[argn], "SEEK_???");
 	}
 	return RVAL_LUDECIMAL;
 }
@@ -612,8 +638,7 @@ struct tcb *tcp;
 
 #ifndef HAVE_LONG_LONG_OFF_T
 int
-sys_truncate(tcp)
-struct tcb *tcp;
+sys_truncate(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
@@ -625,13 +650,11 @@ struct tcb *tcp;
 
 #if _LFS64_LARGEFILE || HAVE_LONG_LONG_OFF_T
 int
-sys_truncate64(tcp)
-struct tcb *tcp;
+sys_truncate64(struct tcb *tcp)
 {
 	if (entering(tcp)) {
-		ALIGN64 (tcp, 1);
 		printpath(tcp, tcp->u_arg[0]);
-		tprintf(", %llu", LONG_LONG(tcp->u_arg[1],tcp->u_arg[2]));
+		printllval(tcp, ", %llu", 1);
 	}
 	return 0;
 }
@@ -639,8 +662,7 @@ struct tcb *tcp;
 
 #ifndef HAVE_LONG_LONG_OFF_T
 int
-sys_ftruncate(tcp)
-struct tcb *tcp;
+sys_ftruncate(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		tprintf("%ld, %lu", tcp->u_arg[0], tcp->u_arg[1]);
@@ -651,13 +673,11 @@ struct tcb *tcp;
 
 #if _LFS64_LARGEFILE || HAVE_LONG_LONG_OFF_T
 int
-sys_ftruncate64(tcp)
-struct tcb *tcp;
+sys_ftruncate64(struct tcb *tcp)
 {
 	if (entering(tcp)) {
-		ALIGN64 (tcp, 1);
-		tprintf("%ld, %llu", tcp->u_arg[0],
-			LONG_LONG(tcp->u_arg[1] ,tcp->u_arg[2]));
+		tprintf("%ld, ", tcp->u_arg[0]);
+		printllval(tcp, "%llu", 1);
 	}
 	return 0;
 }
@@ -677,8 +697,7 @@ static const struct xlat modetypes[] = {
 };
 
 static const char *
-sprintmode(mode)
-int mode;
+sprintmode(int mode)
 {
 	static char buf[64];
 	const char *s;
@@ -722,29 +741,29 @@ sprinttime(time_t t)
 
 #ifdef LINUXSPARC
 typedef struct {
-        int     tv_sec;
-        int     tv_nsec;
+	int     tv_sec;
+	int     tv_nsec;
 } timestruct_t;
 
 struct solstat {
-        unsigned        st_dev;
-        int             st_pad1[3];     /* network id */
-        unsigned        st_ino;
-        unsigned        st_mode;
-        unsigned        st_nlink;
-        unsigned        st_uid;
-        unsigned        st_gid;
-        unsigned        st_rdev;
-        int             st_pad2[2];
-        int             st_size;
-        int             st_pad3;        /* st_size, off_t expansion */
-        timestruct_t    st_atime;
-        timestruct_t    st_mtime;
-        timestruct_t    st_ctime;
-        int             st_blksize;
-        int             st_blocks;
-        char            st_fstype[16];
-        int             st_pad4[8];     /* expansion area */
+	unsigned        st_dev;
+	int             st_pad1[3];     /* network id */
+	unsigned        st_ino;
+	unsigned        st_mode;
+	unsigned        st_nlink;
+	unsigned        st_uid;
+	unsigned        st_gid;
+	unsigned        st_rdev;
+	int             st_pad2[2];
+	int             st_size;
+	int             st_pad3;        /* st_size, off_t expansion */
+	timestruct_t    st_atime;
+	timestruct_t    st_mtime;
+	timestruct_t    st_ctime;
+	int             st_blksize;
+	int             st_blocks;
+	char            st_fstype[16];
+	int             st_pad4[8];     /* expansion area */
 };
 
 static void
@@ -752,14 +771,6 @@ printstatsol(struct tcb *tcp, long addr)
 {
 	struct solstat statbuf;
 
-	if (!addr) {
-		tprintf("NULL");
-		return;
-	}
-	if (syserror(tcp) || !verbose(tcp)) {
-		tprintf("%#lx", addr);
-		return;
-	}
 	if (umove(tcp, addr, &statbuf) < 0) {
 		tprintf("{...}");
 		return;
@@ -800,20 +811,10 @@ printstatsol(struct tcb *tcp, long addr)
 
 #if defined (SPARC64)
 static void
-printstat_sparc64(tcp, addr)
-struct tcb *tcp;
-long addr;
+printstat_sparc64(struct tcb *tcp, long addr)
 {
 	struct stat_sparc64 statbuf;
 
-	if (!addr) {
-		tprintf("NULL");
-		return;
-	}
-	if (syserror(tcp) || !verbose(tcp)) {
-		tprintf("%#lx", addr);
-		return;
-	}
 	if (umove(tcp, addr, &statbuf) < 0) {
 		tprintf("{...}");
 		return;
@@ -882,8 +883,7 @@ static const struct xlat fileflags[] = {
 
 #ifdef FREEBSD
 int
-sys_chflags(tcp)
-struct tcb *tcp;
+sys_chflags(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
@@ -894,8 +894,7 @@ struct tcb *tcp;
 }
 
 int
-sys_fchflags(tcp)
-struct tcb *tcp;
+sys_fchflags(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		tprintf("%ld, ", tcp->u_arg[0]);
@@ -907,49 +906,47 @@ struct tcb *tcp;
 
 #ifndef HAVE_LONG_LONG_OFF_T
 static void
-realprintstat(tcp, statbuf)
-struct tcb *tcp;
-struct stat *statbuf;
+realprintstat(struct tcb *tcp, struct stat *statbuf)
 {
-    if (!abbrev(tcp)) {
-	    tprintf("{st_dev=makedev(%lu, %lu), st_ino=%lu, st_mode=%s, ",
-		    (unsigned long) major(statbuf->st_dev),
-		    (unsigned long) minor(statbuf->st_dev),
-		    (unsigned long) statbuf->st_ino,
-		    sprintmode(statbuf->st_mode));
-	    tprintf("st_nlink=%lu, st_uid=%lu, st_gid=%lu, ",
-		    (unsigned long) statbuf->st_nlink,
-		    (unsigned long) statbuf->st_uid,
-		    (unsigned long) statbuf->st_gid);
+	if (!abbrev(tcp)) {
+		tprintf("{st_dev=makedev(%lu, %lu), st_ino=%lu, st_mode=%s, ",
+			(unsigned long) major(statbuf->st_dev),
+			(unsigned long) minor(statbuf->st_dev),
+			(unsigned long) statbuf->st_ino,
+			sprintmode(statbuf->st_mode));
+		tprintf("st_nlink=%lu, st_uid=%lu, st_gid=%lu, ",
+			(unsigned long) statbuf->st_nlink,
+			(unsigned long) statbuf->st_uid,
+			(unsigned long) statbuf->st_gid);
 #ifdef HAVE_STRUCT_STAT_ST_BLKSIZE
-	    tprintf("st_blksize=%lu, ", (unsigned long) statbuf->st_blksize);
-#endif /* HAVE_STRUCT_STAT_ST_BLKSIZE */
+		tprintf("st_blksize=%lu, ", (unsigned long) statbuf->st_blksize);
+#endif
 #ifdef HAVE_STRUCT_STAT_ST_BLOCKS
-	    tprintf("st_blocks=%lu, ", (unsigned long) statbuf->st_blocks);
-#endif /* HAVE_STRUCT_STAT_ST_BLOCKS */
-    }
-    else
-	    tprintf("{st_mode=%s, ", sprintmode(statbuf->st_mode));
-    switch (statbuf->st_mode & S_IFMT) {
-    case S_IFCHR: case S_IFBLK:
+		tprintf("st_blocks=%lu, ", (unsigned long) statbuf->st_blocks);
+#endif
+	}
+	else
+		tprintf("{st_mode=%s, ", sprintmode(statbuf->st_mode));
+	switch (statbuf->st_mode & S_IFMT) {
+	case S_IFCHR: case S_IFBLK:
 #ifdef HAVE_STRUCT_STAT_ST_RDEV
-	    tprintf("st_rdev=makedev(%lu, %lu), ",
-		    (unsigned long) major(statbuf->st_rdev),
-		    (unsigned long) minor(statbuf->st_rdev));
+		tprintf("st_rdev=makedev(%lu, %lu), ",
+			(unsigned long) major(statbuf->st_rdev),
+			(unsigned long) minor(statbuf->st_rdev));
 #else /* !HAVE_STRUCT_STAT_ST_RDEV */
-	    tprintf("st_size=makedev(%lu, %lu), ",
-		    (unsigned long) major(statbuf->st_size),
-		    (unsigned long) minor(statbuf->st_size));
+		tprintf("st_size=makedev(%lu, %lu), ",
+			(unsigned long) major(statbuf->st_size),
+			(unsigned long) minor(statbuf->st_size));
 #endif /* !HAVE_STRUCT_STAT_ST_RDEV */
-	    break;
-    default:
-	    tprintf("st_size=%lu, ", statbuf->st_size);
-	    break;
-    }
-    if (!abbrev(tcp)) {
-	    tprintf("st_atime=%s, ", sprinttime(statbuf->st_atime));
-	    tprintf("st_mtime=%s, ", sprinttime(statbuf->st_mtime));
-	    tprintf("st_ctime=%s", sprinttime(statbuf->st_ctime));
+		break;
+	default:
+		tprintf("st_size=%lu, ", statbuf->st_size);
+		break;
+	}
+	if (!abbrev(tcp)) {
+		tprintf("st_atime=%s, ", sprinttime(statbuf->st_atime));
+		tprintf("st_mtime=%s, ", sprinttime(statbuf->st_mtime));
+		tprintf("st_ctime=%s", sprinttime(statbuf->st_ctime));
 #if HAVE_STRUCT_STAT_ST_FLAGS
 		tprintf(", st_flags=");
 		printflags(fileflags, statbuf->st_flags, "UF_???");
@@ -968,32 +965,17 @@ struct stat *statbuf;
 		tprintf(", st_gen=%u", statbuf->st_gen);
 #endif
 		tprintf("}");
-    }
-    else
-	    tprintf("...}");
+	}
+	else
+		tprintf("...}");
 }
 
 
 static void
-printstat(tcp, addr)
-struct tcb *tcp;
-long addr;
+printstat(struct tcb *tcp, long addr)
 {
 	struct stat statbuf;
 
-#ifdef LINUXSPARC
- 	if (current_personality == 1) {
- 		printstatsol(tcp, addr);
- 		return;
- 	}
-#ifdef SPARC64
-	else if (current_personality == 2) {
-		printstat_sparc64(tcp, addr);
-		return;
-	}
-#endif
-#endif /* LINUXSPARC */
-
 	if (!addr) {
 		tprintf("NULL");
 		return;
@@ -1002,6 +984,20 @@ long addr;
 		tprintf("%#lx", addr);
 		return;
 	}
+
+#ifdef LINUXSPARC
+	if (current_personality == 1) {
+		printstatsol(tcp, addr);
+		return;
+	}
+#ifdef SPARC64
+	else if (current_personality == 2) {
+		printstat_sparc64(tcp, addr);
+		return;
+	}
+#endif
+#endif /* LINUXSPARC */
+
 	if (umove(tcp, addr, &statbuf) < 0) {
 		tprintf("{...}");
 		return;
@@ -1045,29 +1041,14 @@ struct stat64 {
 
 #ifdef HAVE_STAT64
 static void
-printstat64(tcp, addr)
-struct tcb *tcp;
-long addr;
+printstat64(struct tcb *tcp, long addr)
 {
 	struct stat64 statbuf;
 
-#ifdef	STAT64_SIZE
+#ifdef STAT64_SIZE
 	(void) sizeof(char[sizeof statbuf == STAT64_SIZE ? 1 : -1]);
 #endif
 
-#ifdef LINUXSPARC
- 	if (current_personality == 1) {
- 		printstatsol(tcp, addr);
- 		return;
- 	}
-#ifdef SPARC64
-	else if (current_personality == 2) {
-		printstat_sparc64(tcp, addr);
-		return;
-	}
-#endif
-#endif /* LINUXSPARC */
-
 	if (!addr) {
 		tprintf("NULL");
 		return;
@@ -1076,6 +1057,27 @@ long addr;
 		tprintf("%#lx", addr);
 		return;
 	}
+
+#ifdef LINUXSPARC
+	if (current_personality == 1) {
+		printstatsol(tcp, addr);
+		return;
+	}
+# ifdef SPARC64
+	else if (current_personality == 2) {
+		printstat_sparc64(tcp, addr);
+		return;
+	}
+# endif
+#endif /* LINUXSPARC */
+
+#if defined LINUX && defined X86_64
+	if (current_personality == 0) {
+		printstat(tcp, addr);
+		return;
+	}
+#endif
+
 	if (umove(tcp, addr, &statbuf) < 0) {
 		tprintf("{...}");
 		return;
@@ -1159,41 +1161,30 @@ long addr;
 
 #if defined(LINUX) && defined(HAVE_STRUCT___OLD_KERNEL_STAT)
 static void
-convertoldstat(oldbuf, newbuf)
-const struct __old_kernel_stat *oldbuf;
-struct stat *newbuf;
+convertoldstat(const struct __old_kernel_stat *oldbuf, struct stat *newbuf)
 {
-    newbuf->st_dev=oldbuf->st_dev;
-    newbuf->st_ino=oldbuf->st_ino;
-    newbuf->st_mode=oldbuf->st_mode;
-    newbuf->st_nlink=oldbuf->st_nlink;
-    newbuf->st_uid=oldbuf->st_uid;
-    newbuf->st_gid=oldbuf->st_gid;
-    newbuf->st_rdev=oldbuf->st_rdev;
-    newbuf->st_size=oldbuf->st_size;
-    newbuf->st_atime=oldbuf->st_atime;
-    newbuf->st_mtime=oldbuf->st_mtime;
-    newbuf->st_ctime=oldbuf->st_ctime;
-    newbuf->st_blksize=0;	/* not supported in old_stat */
-    newbuf->st_blocks=0;		/* not supported in old_stat */
+	newbuf->st_dev = oldbuf->st_dev;
+	newbuf->st_ino = oldbuf->st_ino;
+	newbuf->st_mode = oldbuf->st_mode;
+	newbuf->st_nlink = oldbuf->st_nlink;
+	newbuf->st_uid = oldbuf->st_uid;
+	newbuf->st_gid = oldbuf->st_gid;
+	newbuf->st_rdev = oldbuf->st_rdev;
+	newbuf->st_size = oldbuf->st_size;
+	newbuf->st_atime = oldbuf->st_atime;
+	newbuf->st_mtime = oldbuf->st_mtime;
+	newbuf->st_ctime = oldbuf->st_ctime;
+	newbuf->st_blksize = 0; /* not supported in old_stat */
+	newbuf->st_blocks = 0; /* not supported in old_stat */
 }
 
 
 static void
-printoldstat(tcp, addr)
-struct tcb *tcp;
-long addr;
+printoldstat(struct tcb *tcp, long addr)
 {
 	struct __old_kernel_stat statbuf;
 	struct stat newstatbuf;
 
-#ifdef LINUXSPARC
- 	if (current_personality == 1) {
- 		printstatsol(tcp, addr);
- 		return;
- 	}
-#endif /* LINUXSPARC */
-
 	if (!addr) {
 		tprintf("NULL");
 		return;
@@ -1202,6 +1193,14 @@ long addr;
 		tprintf("%#lx", addr);
 		return;
 	}
+
+#ifdef LINUXSPARC
+	if (current_personality == 1) {
+		printstatsol(tcp, addr);
+		return;
+	}
+#endif /* LINUXSPARC */
+
 	if (umove(tcp, addr, &statbuf) < 0) {
 		tprintf("{...}");
 		return;
@@ -1214,8 +1213,7 @@ long addr;
 
 #ifndef HAVE_LONG_LONG_OFF_T
 int
-sys_stat(tcp)
-struct tcb *tcp;
+sys_stat(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
@@ -1228,8 +1226,7 @@ struct tcb *tcp;
 #endif
 
 int
-sys_stat64(tcp)
-struct tcb *tcp;
+sys_stat64(struct tcb *tcp)
 {
 #ifdef HAVE_STAT64
 	if (entering(tcp)) {
@@ -1276,8 +1273,7 @@ sys_newfstatat(struct tcb *tcp)
 
 #if defined(LINUX) && defined(HAVE_STRUCT___OLD_KERNEL_STAT)
 int
-sys_oldstat(tcp)
-struct tcb *tcp;
+sys_oldstat(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
@@ -1291,8 +1287,7 @@ struct tcb *tcp;
 
 #ifndef HAVE_LONG_LONG_OFF_T
 int
-sys_fstat(tcp)
-struct tcb *tcp;
+sys_fstat(struct tcb *tcp)
 {
 	if (entering(tcp))
 		tprintf("%ld, ", tcp->u_arg[0]);
@@ -1304,8 +1299,7 @@ struct tcb *tcp;
 #endif
 
 int
-sys_fstat64(tcp)
-struct tcb *tcp;
+sys_fstat64(struct tcb *tcp)
 {
 #ifdef HAVE_STAT64
 	if (entering(tcp))
@@ -1321,8 +1315,7 @@ struct tcb *tcp;
 
 #if defined(LINUX) && defined(HAVE_STRUCT___OLD_KERNEL_STAT)
 int
-sys_oldfstat(tcp)
-struct tcb *tcp;
+sys_oldfstat(struct tcb *tcp)
 {
 	if (entering(tcp))
 		tprintf("%ld, ", tcp->u_arg[0]);
@@ -1335,8 +1328,7 @@ struct tcb *tcp;
 
 #ifndef HAVE_LONG_LONG_OFF_T
 int
-sys_lstat(tcp)
-struct tcb *tcp;
+sys_lstat(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
@@ -1349,8 +1341,7 @@ struct tcb *tcp;
 #endif
 
 int
-sys_lstat64(tcp)
-struct tcb *tcp;
+sys_lstat64(struct tcb *tcp)
 {
 #ifdef HAVE_STAT64
 	if (entering(tcp)) {
@@ -1367,8 +1358,7 @@ struct tcb *tcp;
 
 #if defined(LINUX) && defined(HAVE_STRUCT___OLD_KERNEL_STAT)
 int
-sys_oldlstat(tcp)
-struct tcb *tcp;
+sys_oldlstat(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
@@ -1384,8 +1374,7 @@ struct tcb *tcp;
 #if defined(SVR4) || defined(LINUXSPARC)
 
 int
-sys_xstat(tcp)
-struct tcb *tcp;
+sys_xstat(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		tprintf("%ld, ", tcp->u_arg[0]);
@@ -1403,8 +1392,7 @@ struct tcb *tcp;
 }
 
 int
-sys_fxstat(tcp)
-struct tcb *tcp;
+sys_fxstat(struct tcb *tcp)
 {
 	if (entering(tcp))
 		tprintf("%ld, %ld, ", tcp->u_arg[0], tcp->u_arg[1]);
@@ -1420,8 +1408,7 @@ struct tcb *tcp;
 }
 
 int
-sys_lxstat(tcp)
-struct tcb *tcp;
+sys_lxstat(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		tprintf("%ld, ", tcp->u_arg[0]);
@@ -1439,8 +1426,7 @@ struct tcb *tcp;
 }
 
 int
-sys_xmknod(tcp)
-struct tcb *tcp;
+sys_xmknod(struct tcb *tcp)
 {
 	int mode = tcp->u_arg[2];
 
@@ -1494,8 +1480,7 @@ static const struct xlat aclcmds[] = {
 };
 
 int
-sys_acl(tcp)
-struct tcb *tcp;
+sys_acl(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
@@ -1516,8 +1501,7 @@ struct tcb *tcp;
 
 
 int
-sys_facl(tcp)
-struct tcb *tcp;
+sys_facl(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		tprintf("%ld, ", tcp->u_arg[0]);
@@ -1551,8 +1535,7 @@ static const struct xlat aclipc[] = {
 
 
 int
-sys_aclipc(tcp)
-struct tcb *tcp;
+sys_aclipc(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		printxval(aclipc, tcp->u_arg[0], "???IPC???");
@@ -1571,8 +1554,6 @@ struct tcb *tcp;
 	return 0;
 }
 
-
-
 #endif /* HAVE_SYS_ACL_H */
 
 #endif /* SVR4 || LINUXSPARC */
@@ -1617,8 +1598,7 @@ static const struct xlat fsmagic[] = {
 #ifndef SVR4
 
 static const char *
-sprintfstype(magic)
-int magic;
+sprintfstype(int magic)
 {
 	static char buf[32];
 #ifdef LINUX
@@ -1635,9 +1615,7 @@ int magic;
 }
 
 static void
-printstatfs(tcp, addr)
-struct tcb *tcp;
-long addr;
+printstatfs(struct tcb *tcp, long addr)
 {
 	struct statfs statbuf;
 
@@ -1680,8 +1658,7 @@ long addr;
 }
 
 int
-sys_statfs(tcp)
-struct tcb *tcp;
+sys_statfs(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
@@ -1693,8 +1670,7 @@ struct tcb *tcp;
 }
 
 int
-sys_fstatfs(tcp)
-struct tcb *tcp;
+sys_fstatfs(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		tprintf("%lu, ", tcp->u_arg[0]);
@@ -1704,11 +1680,9 @@ struct tcb *tcp;
 	return 0;
 }
 
-#ifdef LINUX
+#if defined LINUX && defined HAVE_STATFS64
 static void
-printstatfs64(tcp, addr)
-struct tcb *tcp;
-long addr;
+printstatfs64(struct tcb *tcp, long addr)
 {
 	struct statfs64 statbuf;
 
@@ -1738,8 +1712,7 @@ long addr;
 }
 
 int
-sys_statfs64(tcp)
-struct tcb *tcp;
+sys_statfs64(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
@@ -1754,8 +1727,7 @@ struct tcb *tcp;
 }
 
 int
-sys_fstatfs64(tcp)
-struct tcb *tcp;
+sys_fstatfs64(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		tprintf("%lu, %lu, ", tcp->u_arg[0], tcp->u_arg[1]);
@@ -1772,8 +1744,7 @@ struct tcb *tcp;
 #if defined(LINUX) && defined(__alpha)
 
 int
-osf_statfs(tcp)
-struct tcb *tcp;
+osf_statfs(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
@@ -1786,8 +1757,7 @@ struct tcb *tcp;
 }
 
 int
-osf_fstatfs(tcp)
-struct tcb *tcp;
+osf_fstatfs(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		tprintf("%lu, ", tcp->u_arg[0]);
@@ -1802,10 +1772,8 @@ struct tcb *tcp;
 #endif /* !SVR4 */
 
 #ifdef SUNOS4
-
 int
-sys_ustat(tcp)
-struct tcb *tcp;
+sys_ustat(struct tcb *tcp)
 {
 	struct ustat statbuf;
 
@@ -1832,12 +1800,10 @@ struct tcb *tcp;
 	}
 	return 0;
 }
-
 #endif /* SUNOS4 */
 
 int
-sys_pivotroot(tcp)
-struct tcb *tcp;
+sys_pivotroot(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
@@ -1850,8 +1816,7 @@ struct tcb *tcp;
 
 /* directory */
 int
-sys_chdir(tcp)
-struct tcb *tcp;
+sys_chdir(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
@@ -1886,8 +1851,7 @@ sys_mkdirat(struct tcb *tcp)
 #endif
 
 int
-sys_rmdir(tcp)
-struct tcb *tcp;
+sys_rmdir(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
@@ -1896,8 +1860,7 @@ struct tcb *tcp;
 }
 
 int
-sys_fchdir(tcp)
-struct tcb *tcp;
+sys_fchdir(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		tprintf("%ld", tcp->u_arg[0]);
@@ -1906,8 +1869,7 @@ struct tcb *tcp;
 }
 
 int
-sys_chroot(tcp)
-struct tcb *tcp;
+sys_chroot(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
@@ -1917,8 +1879,7 @@ struct tcb *tcp;
 
 #if defined(SUNOS4) || defined(SVR4)
 int
-sys_fchroot(tcp)
-struct tcb *tcp;
+sys_fchroot(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		tprintf("%ld", tcp->u_arg[0]);
@@ -1928,8 +1889,7 @@ struct tcb *tcp;
 #endif /* SUNOS4 || SVR4 */
 
 int
-sys_link(tcp)
-struct tcb *tcp;
+sys_link(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
@@ -1956,8 +1916,7 @@ sys_linkat(struct tcb *tcp)
 #endif
 
 int
-sys_unlink(tcp)
-struct tcb *tcp;
+sys_unlink(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
@@ -1988,8 +1947,7 @@ sys_unlinkat(struct tcb *tcp)
 #endif
 
 int
-sys_symlink(tcp)
-struct tcb *tcp;
+sys_symlink(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
@@ -2046,8 +2004,7 @@ sys_readlinkat(struct tcb *tcp)
 #endif
 
 int
-sys_rename(tcp)
-struct tcb *tcp;
+sys_rename(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
@@ -2073,8 +2030,7 @@ sys_renameat(struct tcb *tcp)
 #endif
 
 int
-sys_chown(tcp)
-struct tcb *tcp;
+sys_chown(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
@@ -2101,8 +2057,7 @@ sys_fchownat(struct tcb *tcp)
 #endif
 
 int
-sys_fchown(tcp)
-struct tcb *tcp;
+sys_fchown(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		tprintf("%ld", tcp->u_arg[0]);
@@ -2139,8 +2094,7 @@ sys_fchmodat(struct tcb *tcp)
 #endif
 
 int
-sys_fchmod(tcp)
-struct tcb *tcp;
+sys_fchmod(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		tprintf("%ld, %#lo", tcp->u_arg[0], tcp->u_arg[1]);
@@ -2150,15 +2104,14 @@ struct tcb *tcp;
 
 #ifdef ALPHA
 int
-sys_osf_utimes(tcp)
-struct tcb *tcp;
+sys_osf_utimes(struct tcb *tcp)
 {
-    if (entering(tcp)) {
-	printpath(tcp, tcp->u_arg[0]);
-	tprintf(", ");
-	printtv_bitness(tcp, tcp->u_arg[1], BITNESS_32,  0);
-    }
-    return 0;
+	if (entering(tcp)) {
+		printpath(tcp, tcp->u_arg[0]);
+		tprintf(", ");
+		printtv_bitness(tcp, tcp->u_arg[1], BITNESS_32,  0);
+	}
+	return 0;
 }
 #endif
 
@@ -2213,8 +2166,7 @@ sys_utimensat(struct tcb *tcp)
 #endif
 
 int
-sys_utime(tcp)
-struct tcb *tcp;
+sys_utime(struct tcb *tcp)
 {
 	union {
 		long utl[2];
@@ -2294,8 +2246,7 @@ sys_mknodat(struct tcb *tcp)
 
 #ifdef FREEBSD
 int
-sys_mkfifo(tcp)
-struct tcb *tcp;
+sys_mkfifo(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
@@ -2306,8 +2257,7 @@ struct tcb *tcp;
 #endif /* FREEBSD */
 
 int
-sys_fsync(tcp)
-struct tcb *tcp;
+sys_fsync(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		tprintf("%ld", tcp->u_arg[0]);
@@ -2318,9 +2268,7 @@ struct tcb *tcp;
 #ifdef LINUX
 
 static void
-printdir(tcp, addr)
-struct tcb *tcp;
-long addr;
+printdir(struct tcb *tcp, long addr)
 {
 	struct dirent d;
 
@@ -2339,8 +2287,7 @@ long addr;
 }
 
 int
-sys_readdir(tcp)
-struct tcb *tcp;
+sys_readdir(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		tprintf("%lu, ", tcp->u_arg[0]);
@@ -2389,7 +2336,8 @@ sys_getdents(struct tcb *tcp)
 		return 0;
 	}
 	len = tcp->u_rval;
-	if ((buf = malloc(len)) == NULL) {
+	buf = len ? malloc(len) : NULL;
+	if (len && !buf) {
 		tprintf("%#lx, %lu", tcp->u_arg[1], tcp->u_arg[2]);
 		fprintf(stderr, "out of memory\n");
 		return 0;
@@ -2472,7 +2420,8 @@ sys_getdents64(struct tcb *tcp)
 		return 0;
 	}
 	len = tcp->u_rval;
-	if ((buf = malloc(len)) == NULL) {
+	buf = len ? malloc(len) : NULL;
+	if (len && !buf) {
 		tprintf("%#lx, %lu", tcp->u_arg[1], tcp->u_arg[2]);
 		fprintf(stderr, "out of memory\n");
 		return 0;
@@ -2588,41 +2537,38 @@ sys_getdirentries(struct tcb *tcp)
 
 #ifdef LINUX
 int
-sys_getcwd(tcp)
-struct tcb *tcp;
+sys_getcwd(struct tcb *tcp)
 {
-    if (exiting(tcp)) {
-	if (syserror(tcp))
-	    tprintf("%#lx", tcp->u_arg[0]);
-	else
-	    printpathn(tcp, tcp->u_arg[0], tcp->u_rval - 1);
-	tprintf(", %lu", tcp->u_arg[1]);
-    }
-    return 0;
+	if (exiting(tcp)) {
+		if (syserror(tcp))
+			tprintf("%#lx", tcp->u_arg[0]);
+		else
+			printpathn(tcp, tcp->u_arg[0], tcp->u_rval - 1);
+		tprintf(", %lu", tcp->u_arg[1]);
+	}
+	return 0;
 }
 #endif /* LINUX */
 
 #ifdef FREEBSD
 int
-sys___getcwd(tcp)
-struct tcb *tcp;
+sys___getcwd(struct tcb *tcp)
 {
-    if (exiting(tcp)) {
-	if (syserror(tcp))
-	    tprintf("%#lx", tcp->u_arg[0]);
-	else
-	    printpathn(tcp, tcp->u_arg[0], tcp->u_arg[1]);
-	tprintf(", %lu", tcp->u_arg[1]);
-    }
-    return 0;
+	if (exiting(tcp)) {
+		if (syserror(tcp))
+			tprintf("%#lx", tcp->u_arg[0]);
+		else
+			printpathn(tcp, tcp->u_arg[0], tcp->u_arg[1]);
+		tprintf(", %lu", tcp->u_arg[1]);
+	}
+	return 0;
 }
 #endif
 
 #ifdef HAVE_SYS_ASYNCH_H
 
 int
-sys_aioread(tcp)
-struct tcb *tcp;
+sys_aioread(struct tcb *tcp)
 {
 	struct aio_result_t res;
 
@@ -2646,8 +2592,7 @@ struct tcb *tcp;
 }
 
 int
-sys_aiowrite(tcp)
-struct tcb *tcp;
+sys_aiowrite(struct tcb *tcp)
 {
 	struct aio_result_t res;
 
@@ -2671,8 +2616,7 @@ struct tcb *tcp;
 }
 
 int
-sys_aiowait(tcp)
-struct tcb *tcp;
+sys_aiowait(struct tcb *tcp)
 {
 	if (entering(tcp))
 		printtv(tcp, tcp->u_arg[0]);
@@ -2680,8 +2624,7 @@ struct tcb *tcp;
 }
 
 int
-sys_aiocancel(tcp)
-struct tcb *tcp;
+sys_aiocancel(struct tcb *tcp)
 {
 	struct aio_result_t res;
 
@@ -2709,150 +2652,142 @@ static const struct xlat xattrflags[] = {
 };
 
 static void
-print_xattr_val(tcp, failed, arg, insize, size)
-struct tcb *tcp;
-int failed;
-unsigned long arg;
-unsigned long insize, size;
+print_xattr_val(struct tcb *tcp, int failed,
+		unsigned long arg,
+		unsigned long insize,
+		unsigned long size)
 {
-    if (!failed) {
-	unsigned long capacity = 4 * size + 1;
-	unsigned char *buf = (capacity < size) ? NULL : malloc(capacity);
-	if (buf == NULL || /* probably a bogus size argument */
-	    umoven(tcp, arg, size, (char *) &buf[3 * size]) < 0) {
-	    failed = 1;
-	}
-	else {
-	    unsigned char *out = buf;
-	    unsigned char *in = &buf[3 * size];
-	    size_t i;
-	    for (i = 0; i < size; ++i)
-		if (isprint(in[i]))
-		    *out++ = in[i];
-		else {
-#define tohex(n) "0123456789abcdef"[n]
-		    *out++ = '\\';
-		    *out++ = 'x';
-		    *out++ = tohex(in[i] / 16);
-		    *out++ = tohex(in[i] % 16);
+	if (!failed) {
+		unsigned long capacity = 4 * size + 1;
+		unsigned char *buf = (capacity < size) ? NULL : malloc(capacity);
+		if (buf == NULL || /* probably a bogus size argument */
+			umoven(tcp, arg, size, (char *) &buf[3 * size]) < 0) {
+			failed = 1;
 		}
-	    /* Don't print terminating NUL if there is one.  */
-	    if (i > 0 && in[i - 1] == '\0')
-		out -= 4;
-	    *out = '\0';
-	    tprintf(", \"%s\", %ld", buf, insize);
+		else {
+			unsigned char *out = buf;
+			unsigned char *in = &buf[3 * size];
+			size_t i;
+			for (i = 0; i < size; ++i) {
+				if (isprint(in[i]))
+					*out++ = in[i];
+				else {
+#define tohex(n) "0123456789abcdef"[n]
+					*out++ = '\\';
+					*out++ = 'x';
+					*out++ = tohex(in[i] / 16);
+					*out++ = tohex(in[i] % 16);
+				}
+			}
+			/* Don't print terminating NUL if there is one.  */
+			if (i > 0 && in[i - 1] == '\0')
+				out -= 4;
+			*out = '\0';
+			tprintf(", \"%s\", %ld", buf, insize);
+		}
+		free(buf);
 	}
-	free(buf);
-    }
-    if (failed)
-	tprintf(", 0x%lx, %ld", arg, insize);
+	if (failed)
+		tprintf(", 0x%lx, %ld", arg, insize);
 }
 
 int
-sys_setxattr(tcp)
-struct tcb *tcp;
+sys_setxattr(struct tcb *tcp)
 {
-    if (entering(tcp)) {
-	printpath(tcp, tcp->u_arg[0]);
-	tprintf(", ");
-	printstr(tcp, tcp->u_arg[1], -1);
-	print_xattr_val(tcp, 0, tcp->u_arg[2], tcp->u_arg[3], tcp->u_arg[3]);
-	tprintf(", ");
-	printflags(xattrflags, tcp->u_arg[4], "XATTR_???");
-    }
-    return 0;
+	if (entering(tcp)) {
+		printpath(tcp, tcp->u_arg[0]);
+		tprintf(", ");
+		printstr(tcp, tcp->u_arg[1], -1);
+		print_xattr_val(tcp, 0, tcp->u_arg[2], tcp->u_arg[3], tcp->u_arg[3]);
+		tprintf(", ");
+		printflags(xattrflags, tcp->u_arg[4], "XATTR_???");
+	}
+	return 0;
 }
 
 int
-sys_fsetxattr(tcp)
-struct tcb *tcp;
+sys_fsetxattr(struct tcb *tcp)
 {
-    if (entering(tcp)) {
-	tprintf("%ld, ", tcp->u_arg[0]);
-	printstr(tcp, tcp->u_arg[1], -1);
-	print_xattr_val(tcp, 0, tcp->u_arg[2], tcp->u_arg[3], tcp->u_arg[3]);
-	tprintf(", ");
-	printflags(xattrflags, tcp->u_arg[4], "XATTR_???");
-    }
-    return 0;
+	if (entering(tcp)) {
+		tprintf("%ld, ", tcp->u_arg[0]);
+		printstr(tcp, tcp->u_arg[1], -1);
+		print_xattr_val(tcp, 0, tcp->u_arg[2], tcp->u_arg[3], tcp->u_arg[3]);
+		tprintf(", ");
+		printflags(xattrflags, tcp->u_arg[4], "XATTR_???");
+	}
+	return 0;
 }
 
 int
-sys_getxattr(tcp)
-struct tcb *tcp;
+sys_getxattr(struct tcb *tcp)
 {
-    if (entering(tcp)) {
-	printpath(tcp, tcp->u_arg[0]);
-	tprintf(", ");
-	printstr(tcp, tcp->u_arg[1], -1);
-    } else {
-	print_xattr_val(tcp, syserror(tcp), tcp->u_arg[2], tcp->u_arg[3],
-			tcp->u_rval);
-    }
-    return 0;
+	if (entering(tcp)) {
+		printpath(tcp, tcp->u_arg[0]);
+		tprintf(", ");
+		printstr(tcp, tcp->u_arg[1], -1);
+	} else {
+		print_xattr_val(tcp, syserror(tcp), tcp->u_arg[2], tcp->u_arg[3],
+				tcp->u_rval);
+	}
+	return 0;
 }
 
 int
-sys_fgetxattr(tcp)
-struct tcb *tcp;
+sys_fgetxattr(struct tcb *tcp)
 {
-    if (entering(tcp)) {
-	tprintf("%ld, ", tcp->u_arg[0]);
-	printstr(tcp, tcp->u_arg[1], -1);
-    } else {
-	print_xattr_val(tcp, syserror(tcp), tcp->u_arg[2], tcp->u_arg[3],
-			tcp->u_rval);
-    }
-    return 0;
+	if (entering(tcp)) {
+		tprintf("%ld, ", tcp->u_arg[0]);
+		printstr(tcp, tcp->u_arg[1], -1);
+	} else {
+		print_xattr_val(tcp, syserror(tcp), tcp->u_arg[2], tcp->u_arg[3],
+				tcp->u_rval);
+	}
+	return 0;
 }
 
 int
-sys_listxattr(tcp)
-struct tcb *tcp;
+sys_listxattr(struct tcb *tcp)
 {
-    if (entering(tcp)) {
-	printpath(tcp, tcp->u_arg[0]);
-    } else {
-	/* XXX Print value in format */
-	tprintf(", %p, %lu", (void *) tcp->u_arg[1], tcp->u_arg[2]);
-    }
-    return 0;
+	if (entering(tcp)) {
+		printpath(tcp, tcp->u_arg[0]);
+	} else {
+		/* XXX Print value in format */
+		tprintf(", %p, %lu", (void *) tcp->u_arg[1], tcp->u_arg[2]);
+	}
+	return 0;
 }
 
 int
-sys_flistxattr(tcp)
-struct tcb *tcp;
+sys_flistxattr(struct tcb *tcp)
 {
-    if (entering(tcp)) {
-	tprintf("%ld", tcp->u_arg[0]);
-    } else {
-	/* XXX Print value in format */
-	tprintf(", %p, %lu", (void *) tcp->u_arg[1], tcp->u_arg[2]);
-    }
-    return 0;
+	if (entering(tcp)) {
+		tprintf("%ld", tcp->u_arg[0]);
+	} else {
+		/* XXX Print value in format */
+		tprintf(", %p, %lu", (void *) tcp->u_arg[1], tcp->u_arg[2]);
+	}
+	return 0;
 }
 
 int
-sys_removexattr(tcp)
-struct tcb *tcp;
+sys_removexattr(struct tcb *tcp)
 {
-    if (entering(tcp)) {
-	printpath(tcp, tcp->u_arg[0]);
-	tprintf(", ");
-	printstr(tcp, tcp->u_arg[1], -1);
-    }
-    return 0;
+	if (entering(tcp)) {
+		printpath(tcp, tcp->u_arg[0]);
+		tprintf(", ");
+		printstr(tcp, tcp->u_arg[1], -1);
+	}
+	return 0;
 }
 
 int
-sys_fremovexattr(tcp)
-struct tcb *tcp;
+sys_fremovexattr(struct tcb *tcp)
 {
-    if (entering(tcp)) {
-	tprintf("%ld, ", tcp->u_arg[0]);
-	printstr(tcp, tcp->u_arg[1], -1);
-    }
-    return 0;
+	if (entering(tcp)) {
+		tprintf("%ld, ", tcp->u_arg[0]);
+		printstr(tcp, tcp->u_arg[1], -1);
+	}
+	return 0;
 }
 
 
@@ -2869,45 +2804,39 @@ static const struct xlat advise[] = {
 
 #ifdef LINUX
 int
-sys_fadvise64(tcp)
-struct tcb *tcp;
+sys_fadvise64(struct tcb *tcp)
 {
-    if (entering(tcp)) {
-	tprintf("%ld, %lld, %ld, ",
-		tcp->u_arg[0],
-# if defined IA64 || defined X86_64 || defined ALPHA
-		(long long int) tcp->u_arg[1], tcp->u_arg[2]);
-	printxval(advise, tcp->u_arg[3], "POSIX_FADV_???");
-#else
-		LONG_LONG(tcp->u_arg[1], tcp->u_arg[2]), tcp->u_arg[3]);
-	printxval(advise, tcp->u_arg[4], "POSIX_FADV_???");
-#endif
-    }
-    return 0;
+	if (entering(tcp)) {
+		int argn;
+		tprintf("%ld, ", tcp->u_arg[0]);
+		argn = printllval(tcp, "%lld", 1);
+		tprintf(", %ld, ", tcp->u_arg[argn++]);
+		printxval(advise, tcp->u_arg[argn], "POSIX_FADV_???");
+	}
+	return 0;
 }
 #endif
 
 
 int
-sys_fadvise64_64(tcp)
-struct tcb *tcp;
+sys_fadvise64_64(struct tcb *tcp)
 {
-    if (entering(tcp)) {
-	tprintf("%ld, %lld, %lld, ",
-		tcp->u_arg[0],
-#if defined LINUX_MIPSN32
-		tcp->ext_arg[1], tcp->ext_arg[2]);
-	printxval(advise, tcp->u_arg[3], "POSIX_FADV_???");
-#elif defined IA64 || defined X86_64 || defined ALPHA || defined LINUX_MIPSN64
-		(long long int) tcp->u_arg[1], (long long int) tcp->u_arg[2]);
-	printxval(advise, tcp->u_arg[3], "POSIX_FADV_???");
+	if (entering(tcp)) {
+		int argn;
+		tprintf("%ld, ", tcp->u_arg[0]);
+#if defined ARM || defined POWERPC
+		argn = printllval(tcp, "%lld, ", 2);
 #else
-		LONG_LONG(tcp->u_arg[1], tcp->u_arg[2]),
-		LONG_LONG(tcp->u_arg[3], tcp->u_arg[4]));
-	printxval(advise, tcp->u_arg[5], "POSIX_FADV_???");
+		argn = printllval(tcp, "%lld, ", 1);
 #endif
-    }
-    return 0;
+		argn = printllval(tcp, "%lld, ", argn);
+#if defined ARM || defined POWERPC
+		printxval(advise, tcp->u_arg[1], "POSIX_FADV_???");
+#else
+		printxval(advise, tcp->u_arg[argn], "POSIX_FADV_???");
+#endif
+	}
+	return 0;
 }
 
 #ifdef LINUX
@@ -2955,16 +2884,23 @@ sys_inotify_rm_watch(struct tcb *tcp)
 	return 0;
 }
 
+int
+sys_inotify_init1(struct tcb *tcp)
+{
+	if (entering(tcp))
+		printflags(open_mode_flags, tcp->u_arg[0], "O_???");
+	return 0;
+}
+
 int
 sys_fallocate(struct tcb *tcp)
 {
 	if (entering(tcp)) {
+		int argn;
 		tprintf("%ld, ", tcp->u_arg[0]);	/* fd */
 		tprintf("%#lo, ", tcp->u_arg[1]);	/* mode */
-		tprintf("%llu, ", LONG_LONG(tcp->u_arg[2],
-			tcp->u_arg[3]));		/* offset */
-		tprintf("%llu", LONG_LONG(tcp->u_arg[4],
-			tcp->u_arg[5]));		/* len */
+		argn = printllval(tcp, "%llu, ", 2);	/* offset */
+		printllval(tcp, "%llu", argn);		/* len */
 	}
 	return 0;
 }
