@@ -168,7 +168,7 @@
 	{ 1,	0,	sys_sched_get_priority_min,"sched_get_priority_min"}, /* 160 */
 	{ 2,	0,	sys_sched_rr_get_interval,"sched_rr_get_interval"}, /* 161 */
 	{ 2,	0,	sys_nanosleep,		"nanosleep"		}, /* 162 */
-	{ 4,	0,	sys_mremap,		"mremap"		}, /* 163 */
+	{ 5,	0,	sys_mremap,		"mremap"		}, /* 163 */
 	{ 3,	0,	sys_setresuid,		"setresuid"		}, /* 164 */
 	{ 3,	0,	sys_setresuid,		"getresuid"		}, /* 165 */
 	{ 2,	TS,	sys_sigaltstack,	"sigaltstack"		}, /* 166 */
@@ -314,13 +314,13 @@
 	{ 2,	TD,	sys_timerfd_create,	"timerfd_create"	}, /* 306 */
 	{ 4,	TD,	sys_timerfd_settime,	"timerfd_settime"	}, /* 307 */
 	{ 2,	TD,	sys_timerfd_gettime,	"timerfd_gettime"	}, /* 308 */
-	{ 4,	TD|TS,	printargs,		"signalfd4"		}, /* 309 */
-	{ 2,	TD,	printargs,		"eventfd2"		}, /* 310 */
-	{ 1,	0,	printargs,		"epoll_create1"		}, /* 311 */
-	{ 3,	TD,	printargs,		"dup3"			}, /* 312 */
-	{ 2,	TD,	printargs,		"pipe2"			}, /* 313 */
-	{ 1,	TD,	printargs,		"inotify_init1"		}, /* 314 */
+	{ 4,	TD|TS,	sys_signalfd4,		"signalfd4"		}, /* 309 */
+	{ 2,	TD,	sys_eventfd2,		"eventfd2"		}, /* 310 */
+	{ 1,	0,	sys_epoll_create1,	"epoll_create1"		}, /* 311 */
+	{ 3,	TD,	sys_dup3,		"dup3"			}, /* 312 */
+	{ 2,	TD,	sys_pipe2,		"pipe2"			}, /* 313 */
+	{ 1,	TD,	sys_inotify_init1,	"inotify_init1"		}, /* 314 */
 	{ 5,	TD,	printargs,		"preadv"		}, /* 315 */
 	{ 5,	TD,	printargs,		"pwritev"		}, /* 316 */
 	{ 4,	TP|TS,	printargs,		"rt_tgsigqueueinfo"	}, /* 317 */
-	{ 5,	0,	printargs,		"perf_counter_open"	}, /* 318 */
+	{ 5,	TD,	printargs,		"perf_event_open"	}, /* 318 */
