@@ -27,7 +27,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: net.c,v 1.52 2006/12/13 16:59:44 ldv Exp $
+ *	$Id$
  */
 
 #include "defs.h"
@@ -320,17 +320,32 @@ static const struct xlat socktypes[] = {
 #ifdef SOCK_RAW
 	{ SOCK_RAW,	"SOCK_RAW"	},
 #endif
+#ifdef SOCK_RDM
+	{ SOCK_RDM,	"SOCK_RDM"	},
+#endif
 #ifdef SOCK_SEQPACKET
 	{ SOCK_SEQPACKET,"SOCK_SEQPACKET"},
 #endif
-#ifdef SOCK_RDM
-	{ SOCK_RDM,	"SOCK_RDM"	},
+#ifdef SOCK_DCCP
+	{ SOCK_DCCP,	"SOCK_DCCP"	},
 #endif
 #ifdef SOCK_PACKET
 	{ SOCK_PACKET,	"SOCK_PACKET"	},
 #endif
 	{ 0,		NULL		},
 };
+const struct xlat sock_type_flags[] = {
+#ifdef SOCK_CLOEXEC
+	{ SOCK_CLOEXEC,	"SOCK_CLOEXEC"	},
+#endif
+#ifdef SOCK_NONBLOCK
+	{ SOCK_NONBLOCK,"SOCK_NONBLOCK"	},
+#endif
+	{ 0,		NULL		},
+};
+#ifndef SOCK_TYPE_MASK
+# define SOCK_TYPE_MASK 0xf
+#endif
 static const struct xlat socketlayers[] = {
 #if defined(SOL_IP)
 	{ SOL_IP,	"SOL_IP"	},
@@ -437,59 +452,62 @@ static const struct xlat protocols[] = {
 	{ 0,		NULL		},
 };
 static const struct xlat msg_flags[] = {
-	{ MSG_OOB,	"MSG_OOB"	},
+	{ MSG_OOB,		"MSG_OOB"		},
 #ifdef MSG_DONTROUTE
-	{ MSG_DONTROUTE,"MSG_DONTROUTE"	},
+	{ MSG_DONTROUTE,	"MSG_DONTROUTE"		},
 #endif
 #ifdef MSG_PEEK
-	{ MSG_PEEK,	"MSG_PEEK"	},
+	{ MSG_PEEK,		"MSG_PEEK"		},
 #endif
 #ifdef MSG_CTRUNC
-	{ MSG_CTRUNC,	"MSG_CTRUNC"	},
+	{ MSG_CTRUNC,		"MSG_CTRUNC"		},
 #endif
 #ifdef MSG_PROXY
-	{ MSG_PROXY,	"MSG_PROXY"	},
+	{ MSG_PROXY,		"MSG_PROXY"		},
 #endif
 #ifdef MSG_EOR
-	{ MSG_EOR,	"MSG_EOR"	},
+	{ MSG_EOR,		"MSG_EOR"		},
 #endif
 #ifdef MSG_WAITALL
-	{ MSG_WAITALL,	"MSG_WAITALL"	},
+	{ MSG_WAITALL,		"MSG_WAITALL"		},
 #endif
 #ifdef MSG_TRUNC
-	{ MSG_TRUNC,	"MSG_TRUNC"	},
+	{ MSG_TRUNC,		"MSG_TRUNC"		},
 #endif
 #ifdef MSG_CTRUNC
-	{ MSG_CTRUNC,	"MSG_CTRUNC"	},
+	{ MSG_CTRUNC,		"MSG_CTRUNC"		},
 #endif
 #ifdef MSG_ERRQUEUE
-	{ MSG_ERRQUEUE,	"MSG_ERRQUEUE"	},
+	{ MSG_ERRQUEUE,		"MSG_ERRQUEUE"		},
 #endif
 #ifdef MSG_DONTWAIT
-	{ MSG_DONTWAIT,	"MSG_DONTWAIT"	},
+	{ MSG_DONTWAIT,		"MSG_DONTWAIT"		},
 #endif
 #ifdef MSG_CONFIRM
-	{ MSG_CONFIRM,	"MSG_CONFIRM"	},
+	{ MSG_CONFIRM,		"MSG_CONFIRM"		},
 #endif
 #ifdef MSG_PROBE
-	{ MSG_PROBE,	"MSG_PROBE"	},
+	{ MSG_PROBE,		"MSG_PROBE"		},
 #endif
 #ifdef MSG_FIN
-	{ MSG_FIN,	"MSG_FIN"	},
+	{ MSG_FIN,		"MSG_FIN"		},
 #endif
 #ifdef MSG_SYN
-	{ MSG_SYN,	"MSG_SYN"	},
+	{ MSG_SYN,		"MSG_SYN"		},
 #endif
 #ifdef MSG_RST
-	{ MSG_RST,	"MSG_RST"	},
+	{ MSG_RST,		"MSG_RST"		},
 #endif
 #ifdef MSG_NOSIGNAL
-	{ MSG_NOSIGNAL,	"MSG_NOSIGNAL"	},
+	{ MSG_NOSIGNAL,		"MSG_NOSIGNAL"		},
 #endif
 #ifdef MSG_MORE
-	{ MSG_MORE,	"MSG_MORE"	},
+	{ MSG_MORE,		"MSG_MORE"		},
 #endif
-	{ 0,		NULL		},
+#ifdef MSG_CMSG_CLOEXEC
+	{ MSG_CMSG_CLOEXEC,	"MSG_CMSG_CLOEXEC"	},
+#endif
+	{ 0,			NULL			},
 };
 
 static const struct xlat sockoptions[] = {
@@ -800,8 +818,12 @@ static const struct xlat sockrawoptions[] = {
 
 #ifdef SOL_PACKET
 static const struct xlat sockpacketoptions[] = {
+#ifdef PACKET_ADD_MEMBERSHIP
 	{ PACKET_ADD_MEMBERSHIP,	"PACKET_ADD_MEMBERSHIP"	},
+#endif
+#ifdef PACKET_DROP_MEMBERSHIP
 	{ PACKET_DROP_MEMBERSHIP,	"PACKET_DROP_MEMBERSHIP"},
+#endif
 #if defined(PACKET_RECV_OUTPUT)
 	{ PACKET_RECV_OUTPUT,		"PACKET_RECV_OUTPUT"	},
 #endif
@@ -810,6 +832,30 @@ static const struct xlat sockpacketoptions[] = {
 #endif
 #if defined(PACKET_STATISTICS)
 	{ PACKET_STATISTICS,		"PACKET_STATISTICS"	},
+#endif
+#if defined(PACKET_COPY_THRESH)
+	{ PACKET_COPY_THRESH,		"PACKET_COPY_THRESH"	},
+#endif
+#if defined(PACKET_AUXDATA)
+	{ PACKET_AUXDATA,		"PACKET_AUXDATA"	},
+#endif
+#if defined(PACKET_ORIGDEV)
+	{ PACKET_ORIGDEV,		"PACKET_ORIGDEV"	},
+#endif
+#if defined(PACKET_VERSION)
+	{ PACKET_VERSION,		"PACKET_VERSION"	},
+#endif
+#if defined(PACKET_HDRLEN)
+	{ PACKET_HDRLEN,		"PACKET_HDRLEN"	},
+#endif
+#if defined(PACKET_RESERVE)
+	{ PACKET_RESERVE,		"PACKET_RESERVE"	},
+#endif
+#if defined(PACKET_TX_RING)
+	{ PACKET_TX_RING,		"PACKET_TX_RING"	},
+#endif
+#if defined(PACKET_LOSS)
+	{ PACKET_LOSS,			"PACKET_LOSS"	},
 #endif
 	{ 0,				NULL			},
 };
@@ -934,10 +980,7 @@ static const struct xlat af_packet_types[] = {
 
 
 void
-printsock(tcp, addr, addrlen)
-struct tcb *tcp;
-long addr;
-int addrlen;
+printsock(struct tcb *tcp, long addr, int addrlen)
 {
 	union {
 		char pad[128];
@@ -967,13 +1010,16 @@ int addrlen;
 		tprintf("%#lx", addr);
 		return;
 	}
-	if ((addrlen<2) || (addrlen>sizeof(addrbuf)))
-		addrlen=sizeof(addrbuf);
 
-	if (umoven(tcp, addr, addrlen, (char*)&addrbuf) < 0) {
+	if (addrlen < 2 || addrlen > sizeof(addrbuf))
+		addrlen = sizeof(addrbuf);
+
+	memset(&addrbuf, 0, sizeof(addrbuf));
+	if (umoven(tcp, addr, addrlen, addrbuf.pad) < 0) {
 		tprintf("{...}");
 		return;
 	}
+	addrbuf.pad[sizeof(addrbuf.pad) - 1] = '\0';
 
 	tprintf("{sa_family=");
 	printxval(addrfams, addrbuf.sa.sa_family, "AF_???");
@@ -981,12 +1027,14 @@ int addrlen;
 
 	switch (addrbuf.sa.sa_family) {
 	case AF_UNIX:
-		if (addrlen==2) {
-			tprintf("<nil>");
+		if (addrlen == 2) {
+			tprintf("NULL");
 		} else if (addrbuf.sau.sun_path[0]) {
-			tprintf("path=\"%-.*s\"", addrlen-2, addrbuf.sau.sun_path);
+			tprintf("path=");
+			printpathn(tcp, addr + 2, strlen(addrbuf.sau.sun_path));
 		} else {
-			tprintf("path=@%-.*s", addrlen-3, addrbuf.sau.sun_path+1);
+			tprintf("path=@");
+			printpathn(tcp, addr + 3, strlen(addrbuf.sau.sun_path + 1));
 		}
 		break;
 	case AF_INET:
@@ -1086,10 +1134,7 @@ static const struct xlat scmvals[] = {
 };
 
 static void
-printcmsghdr(tcp, addr, len)
-struct tcb *tcp;
-unsigned long addr;
-unsigned long len;
+printcmsghdr(struct tcb *tcp, unsigned long addr, unsigned long len)
 {
 	struct cmsghdr *cmsg = len < sizeof(struct cmsghdr) ?
 			       NULL : malloc(len);
@@ -1099,7 +1144,7 @@ unsigned long len;
 		return;
 	}
 
-	tprintf(", {cmsg_len=%zu, cmsg_level=", cmsg->cmsg_len);
+	tprintf(", {cmsg_len=%u, cmsg_level=", (unsigned) cmsg->cmsg_len);
 	printxval(socketlayers, cmsg->cmsg_level, "SOL_???");
 	tprintf(", cmsg_type=");
 
@@ -1139,6 +1184,30 @@ unsigned long len;
 	tprintf(", ...}");
 }
 
+static void
+do_msghdr(struct tcb *tcp, struct msghdr *msg)
+{
+	tprintf("{msg_name(%d)=", msg->msg_namelen);
+	printsock(tcp, (long)msg->msg_name, msg->msg_namelen);
+
+	tprintf(", msg_iov(%lu)=", (unsigned long)msg->msg_iovlen);
+	tprint_iov(tcp, (unsigned long)msg->msg_iovlen,
+		   (unsigned long)msg->msg_iov);
+
+#ifdef HAVE_STRUCT_MSGHDR_MSG_CONTROL
+	tprintf(", msg_controllen=%lu", (unsigned long)msg->msg_controllen);
+	if (msg->msg_controllen)
+		printcmsghdr(tcp, (unsigned long) msg->msg_control,
+			     msg->msg_controllen);
+	tprintf(", msg_flags=");
+	printflags(msg_flags, msg->msg_flags, "MSG_???");
+#else /* !HAVE_STRUCT_MSGHDR_MSG_CONTROL */
+	tprintf("msg_accrights=%#lx, msg_accrightslen=%u",
+		(unsigned long) msg->msg_accrights, msg->msg_accrightslen);
+#endif /* !HAVE_STRUCT_MSGHDR_MSG_CONTROL */
+	tprintf("}");
+}
+
 static void
 printmsghdr(tcp, addr)
 struct tcb *tcp;
@@ -1150,37 +1219,57 @@ long addr;
 		tprintf("%#lx", addr);
 		return;
 	}
-	tprintf("{msg_name(%d)=", msg.msg_namelen);
-	printsock(tcp, (long)msg.msg_name, msg.msg_namelen);
-
-	tprintf(", msg_iov(%lu)=", (unsigned long)msg.msg_iovlen);
-	tprint_iov(tcp, (unsigned long)msg.msg_iovlen,
-		   (unsigned long)msg.msg_iov);
-
-#ifdef HAVE_STRUCT_MSGHDR_MSG_CONTROL
-	tprintf(", msg_controllen=%lu", (unsigned long)msg.msg_controllen);
-	if (msg.msg_controllen)
-		printcmsghdr(tcp, (unsigned long) msg.msg_control,
-			     msg.msg_controllen);
-	tprintf(", msg_flags=");
-	printflags(msg_flags, msg.msg_flags, "MSG_???");
-#else /* !HAVE_STRUCT_MSGHDR_MSG_CONTROL */
-	tprintf("msg_accrights=%#lx, msg_accrightslen=%u",
-		(unsigned long) msg.msg_accrights, msg.msg_accrightslen);
-#endif /* !HAVE_STRUCT_MSGHDR_MSG_CONTROL */
-	tprintf("}");
+	do_msghdr(tcp, &msg);
 }
 
+#ifdef LINUX
+static void
+printmmsghdr(struct tcb *tcp, long addr)
+{
+	struct mmsghdr {
+		struct msghdr msg_hdr;
+		unsigned msg_len;
+	} mmsg;
+
+	if (umove(tcp, addr, &mmsg) < 0) {
+		tprintf("%#lx", addr);
+		return;
+	}
+	tprintf("{");
+	do_msghdr(tcp, &mmsg.msg_hdr);
+	tprintf(", %u}", mmsg.msg_len);
+}
+#endif
+
 #endif /* HAVE_SENDMSG */
 
+/*
+ * low bits of the socket type define real socket type,
+ * other bits are socket type flags.
+ */
+static void
+tprint_sock_type(struct tcb *tcp, int flags)
+{
+	const char *str = xlookup(socktypes, flags & SOCK_TYPE_MASK);
+
+	if (str)
+	{
+		tprintf("%s", str);
+		flags &= ~SOCK_TYPE_MASK;
+		if (!flags)
+			return;
+		tprintf("|");
+	}
+	printflags(sock_type_flags, flags, "SOCK_???");
+}
+
 int
-sys_socket(tcp)
-struct tcb *tcp;
+sys_socket(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		printxval(domains, tcp->u_arg[0], "PF_???");
 		tprintf(", ");
-		printxval(socktypes, tcp->u_arg[1], "SOCK_???");
+		tprint_sock_type(tcp, tcp->u_arg[1]);
 		tprintf(", ");
 		switch (tcp->u_arg[0]) {
 		case PF_INET:
@@ -1227,7 +1316,7 @@ sys_so_socketpair(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-	  	/* not sure what this arg is */
+		/* not sure what this arg is */
 		tprintf("0x%lx", tcp->u_arg[0]);
 	}
 	return 0;
@@ -1263,26 +1352,48 @@ struct tcb *tcp;
 	return 0;
 }
 
-int
-sys_accept(tcp)
-struct tcb *tcp;
+static int
+do_accept(struct tcb *tcp, int flags_arg)
 {
 	if (entering(tcp)) {
 		tprintf("%ld, ", tcp->u_arg[0]);
-	} else if (!tcp->u_arg[2])
+		return 0;
+	}
+	if (!tcp->u_arg[2])
 		tprintf("%#lx, NULL", tcp->u_arg[1]);
 	else {
-		if (tcp->u_arg[1] == 0 || syserror(tcp)) {
+		int len;
+		if (tcp->u_arg[1] == 0 || syserror(tcp)
+		    || umove (tcp, tcp->u_arg[2], &len) < 0) {
 			tprintf("%#lx", tcp->u_arg[1]);
 		} else {
-			printsock(tcp, tcp->u_arg[1], tcp->u_arg[2]);
+			printsock(tcp, tcp->u_arg[1], len);
 		}
 		tprintf(", ");
-		printnum(tcp, tcp->u_arg[2], "%lu");
+		printnum_int(tcp, tcp->u_arg[2], "%u");
+	}
+	if (flags_arg >= 0) {
+		tprintf(", ");
+		printflags(sock_type_flags, tcp->u_arg[flags_arg],
+			   "SOCK_???");
 	}
 	return 0;
 }
 
+int
+sys_accept(struct tcb *tcp)
+{
+	return do_accept(tcp, -1);
+}
+
+#ifdef LINUX
+int
+sys_accept4(struct tcb *tcp)
+{
+	return do_accept(tcp, 3);
+}
+#endif
+
 int
 sys_send(tcp)
 struct tcb *tcp;
@@ -1417,6 +1528,55 @@ struct tcb *tcp;
 	return 0;
 }
 
+#ifdef LINUX
+int
+sys_recvmmsg(struct tcb *tcp)
+{
+	static char str[128];
+	if (entering(tcp)) {
+
+		tprintf("%ld, ", tcp->u_arg[0]);
+		if (verbose(tcp)) {
+			sprint_timespec(str, tcp, tcp->u_arg[4]);
+			tcp->auxstr = strdup(str);
+		} else {
+			tprintf("%#lx, %ld, ", tcp->u_arg[1], tcp->u_arg[2]);
+			printflags(msg_flags, tcp->u_arg[3], "MSG_???");
+			tprintf(", ");
+			print_timespec(tcp, tcp->u_arg[4]);
+		}
+		return 0;
+	} else {
+		if (verbose(tcp)) {
+			if (syserror(tcp))
+				tprintf("%#lx", tcp->u_arg[1]);
+			else
+				printmmsghdr(tcp, tcp->u_arg[1]);
+			tprintf(", %ld, ", tcp->u_arg[2]);
+			/* flags */
+			printflags(msg_flags, tcp->u_arg[3], "MSG_???");
+			/* timeout on entrance */
+			tprintf(", %s", tcp->auxstr ? tcp->auxstr : "{...}");
+			free((void *) tcp->auxstr);
+			tcp->auxstr = NULL;
+		}
+		if (syserror(tcp))
+			return 0;
+		if (tcp->u_rval == 0) {
+			tcp->auxstr = "Timeout";
+			return RVAL_STR;
+		}
+		if (!verbose(tcp))
+			return 0;
+		/* timeout on exit */
+		strcpy(str, "left ");
+		sprint_timespec(str + strlen(str), tcp, tcp->u_arg[4]);
+		tcp->auxstr = str;
+		return RVAL_STR;
+	}
+}
+#endif
+
 #endif /* HAVE_SENDMSG */
 
 int
@@ -1454,34 +1614,50 @@ struct tcb *tcp;
 	return sys_accept(tcp);
 }
 
-int
-sys_pipe(tcp)
-struct tcb *tcp;
+static int
+do_pipe(struct tcb *tcp, int flags_arg)
 {
-
-#if defined(LINUX) && !defined(SPARC) && !defined(SPARC64) && !defined(SH) && !defined(IA64)
-	int fds[2];
-
 	if (exiting(tcp)) {
 		if (syserror(tcp)) {
 			tprintf("%#lx", tcp->u_arg[0]);
-			return 0;
-		}
-		if (umoven(tcp, tcp->u_arg[0], sizeof fds, (char *) fds) < 0)
-			tprintf("[...]");
-		else
-			tprintf("[%u, %u]", fds[0], fds[1]);
-	}
+		} else {
+#if defined(LINUX) && !defined(SPARC) && !defined(SPARC64) && !defined(SH) && !defined(IA64)
+			int fds[2];
+
+			if (umoven(tcp, tcp->u_arg[0], sizeof fds, (char *) fds) < 0)
+				tprintf("[...]");
+			else
+				tprintf("[%u, %u]", fds[0], fds[1]);
 #elif defined(SPARC) || defined(SPARC64) || defined(SH) || defined(SVR4) || defined(FREEBSD) || defined(IA64)
-	if (exiting(tcp))
-		tprintf("[%lu, %lu]", tcp->u_rval, getrval2(tcp));
+			tprintf("[%lu, %lu]", tcp->u_rval, getrval2(tcp));
+#else
+			tprintf("%#lx", tcp->u_arg[0]);
 #endif
+		}
+		if (flags_arg >= 0) {
+			tprintf(", ");
+			printflags(open_mode_flags, tcp->u_arg[flags_arg], "O_???");
+		}
+	}
 	return 0;
 }
 
 int
-sys_socketpair(tcp)
-struct tcb *tcp;
+sys_pipe(struct tcb *tcp)
+{
+	return do_pipe(tcp, -1);
+}
+
+#ifdef LINUX
+int
+sys_pipe2(struct tcb *tcp)
+{
+	return do_pipe(tcp, 1);
+}
+#endif
+
+int
+sys_socketpair(struct tcb *tcp)
 {
 #ifdef LINUX
 	int fds[2];
@@ -1490,7 +1666,7 @@ struct tcb *tcp;
 	if (entering(tcp)) {
 		printxval(domains, tcp->u_arg[0], "PF_???");
 		tprintf(", ");
-		printxval(socktypes, tcp->u_arg[1], "SOCK_???");
+		tprint_sock_type(tcp, tcp->u_arg[1]);
 		tprintf(", ");
 		switch (tcp->u_arg[0]) {
 		case PF_INET:
@@ -1527,8 +1703,7 @@ struct tcb *tcp;
 }
 
 int
-sys_getsockopt(tcp)
-struct tcb *tcp;
+sys_getsockopt(struct tcb *tcp)
 {
 	if (entering(tcp)) {
 		tprintf("%ld, ", tcp->u_arg[0]);
@@ -1570,10 +1745,11 @@ struct tcb *tcp;
 			tprintf("%lu", tcp->u_arg[2]);
 			break;
 		}
+		tprintf (", ");
 	} else {
 		int len;
 		if (syserror(tcp) || umove (tcp, tcp->u_arg[4], &len) < 0) {
-			tprintf(", %#lx, %#lx",
+			tprintf("%#lx, %#lx",
 				tcp->u_arg[3], tcp->u_arg[4]);
 			return 0;
 		}
@@ -1583,13 +1759,13 @@ struct tcb *tcp;
 			switch (tcp->u_arg[2]) {
 #ifdef SO_LINGER
 			case SO_LINGER:
-			        if (len == sizeof (struct linger)) {
+				if (len == sizeof (struct linger)) {
 					struct linger linger;
 					if (umove (tcp,
 						   tcp->u_arg[3],
 						   &linger) < 0)
 						break;
-					tprintf(", {onoff=%d, linger=%d}, "
+					tprintf("{onoff=%d, linger=%d}, "
 						"[%d]",
 						linger.l_onoff,
 						linger.l_linger,
@@ -1597,14 +1773,34 @@ struct tcb *tcp;
 					return 0;
 				}
 				break;
+#endif
+			}
+			break;
+		case SOL_PACKET:
+			switch (tcp->u_arg[2]) {
+#ifdef PACKET_STATISTICS
+			case PACKET_STATISTICS:
+				if (len == sizeof(struct tpacket_stats)) {
+					struct tpacket_stats stats;
+					if (umove (tcp,
+						   tcp->u_arg[3],
+						   &stats) < 0)
+						break;
+					tprintf("{packets=%u, drops=%u}, "
+						"[%d]",
+						stats.tp_packets,
+						stats.tp_drops,
+						len);
+					return 0;
+				}
+				break;
 #endif
 			}
 			break;
 		}
 
-		tprintf (", ");
 		if (len == sizeof (int)) {
-			printnum(tcp, tcp->u_arg[3], "%ld");
+			printnum_int(tcp, tcp->u_arg[3], "%d");
 		}
 		else {
 			printstr (tcp, tcp->u_arg[3], len);
@@ -1688,6 +1884,28 @@ int len;
 	    case SOL_PACKET:
 		printxval(sockpacketoptions, name, "PACKET_???");
 		/* TODO: decode packate_mreq for PACKET_*_MEMBERSHIP */
+		switch (name) {
+#ifdef PACKET_RX_RING
+		    case PACKET_RX_RING:
+#endif
+#ifdef PACKET_TX_RING
+		    case PACKET_TX_RING:
+#endif
+#if defined(PACKET_RX_RING) || defined(PACKET_TX_RING)
+			if (len == sizeof(struct tpacket_req)) {
+				struct tpacket_req req;
+				if (umove(tcp, addr, &req) < 0)
+					break;
+				tprintf(", {block_size=%u, block_nr=%u, frame_size=%u, frame_nr=%u}",
+					req.tp_block_size,
+					req.tp_block_nr,
+					req.tp_frame_size,
+					req.tp_frame_nr);
+				return 0;
+			}
+			break;
+#endif /* PACKET_RX_RING || PACKET_TX_RING */
+		}
 		break;
 #endif
 #ifdef SOL_TCP
@@ -1901,17 +2119,6 @@ struct tcb *tcp;
 
 }
 
-#if 0
-
-int
-sys_xsetsockaddr(tcp)
-struct tcb *tcp;
-{
-	return netlib_call (tcp, sys_setsockaddr);
-}
-
-#endif
-
 int
 sys_xgetsockopt(tcp)
 struct tcb *tcp;
@@ -1933,4 +2140,4 @@ struct tcb *tcp;
 	return netlib_call (tcp, sys_shutdown);
 }
 
-#endif
+#endif /* UNIXWARE */
